@@ -92,7 +92,8 @@ void nautilus_file_operations_unmount_mount (GtkWindow                      *par
 					     gboolean                        eject,
 					     gboolean                        check_trash);
 void nautilus_file_operations_mount_volume  (GtkWindow                      *parent_window,
-					     GVolume                        *volume);
+					     GVolume                        *volume,
+					     gboolean                        allow_autorun);
 								
 
 void nautilus_file_operations_copy      (GList                *files,
